/* eslint-disable max-lines */
module.exports = {
  parser: require.resolve('@babel/eslint-parser'),
  parserOptions: {
    babelOptions: {
      babelrc: false,
      configFile: false,
      presets: [
        require.resolve('@babel/preset-env'),
        require.resolve('@babel/preset-react'),
        require.resolve('@babel/preset-typescript'),
      ],
      plugins: [
        // https://babeljs.io/docs/en/babel-plugin-proposal-decorators
        [
          require.resolve('@babel/plugin-proposal-decorators'),
          { version: 'legacy' },
        ],
        // https://babeljs.io/docs/en/babel-plugin-proposal-export-default-from
        require.resolve('@babel/plugin-proposal-export-default-from'),
      ],
    },
    ecmaVersion: 2021,
    ecmaFeatures: {
      impliedStrict: true,
    },
    requireConfigFile: false,
    sourceType: 'module',
  },
  root: true,
  env: {
    browser: true,
    es2021: true,
  },
  ignorePatterns: ['**/node_modules/**', '**/dist/**'],
  plugins: [
    // https://github.com/babel/babel/tree/main/eslint/babel-eslint-plugin
    '@babel/eslint-plugin',
    // https://mysticatea.github.io/eslint-plugin-eslint-comments
    'eslint-plugin-eslint-comments',
    // https://github.com/selaux/eslint-plugin-filenames
    'eslint-plugin-filenames',
    // https://github.com/import-js/eslint-plugin-import
    'eslint-plugin-import',
    // https://github.com/lydell/eslint-plugin-simple-import-sort
    'eslint-plugin-simple-import-sort',
    // https://github.com/xjamundx/eslint-plugin-promise
    'eslint-plugin-promise',
    // https://github.com/prettier/eslint-plugin-prettier
    'eslint-plugin-prettier',
  ],
  extends: [
    'eslint:recommended',
    'plugin:eslint-plugin-eslint-comments/recommended',
    'plugin:eslint-plugin-import/recommended',
    'plugin:eslint-plugin-promise/recommended',
    'plugin:eslint-plugin-prettier/recommended',
  ],
  rules: {
    'prettier/prettier': [
      'error',
      {
        singleQuote: true,
        trailingComma: 'all',
      },
      {
        usePrettierrc: false,
      },
    ],

    // Possible Problems
    'array-callback-return': 'error',
    'no-await-in-loop': 'warn',
    'no-constructor-return': 'error',
    'no-duplicate-imports': 'error',
    'no-promise-executor-return': 'error',
    'no-self-compare': 'error',
    'no-template-curly-in-string': 'error',
    'no-unmodified-loop-condition': 'error',
    'no-unreachable-loop': 'error',
    'no-use-before-define': [
      'error',
      { functions: false, classes: false, variables: false },
    ],
    'require-atomic-updates': 'error',

    // Suggestions
    camelcase: [
      'error',
      {
        allow: ['^UNSAFE_'],
        properties: 'never',
        ignoreGlobals: true,
      },
    ],
    'default-param-last': 'error',
    'dot-notation': 'warn',
    eqeqeq: ['error', 'always', { null: 'ignore' }],
    'guard-for-in': 'error',
    'max-lines': ['warn', { max: 300 }],
    'max-lines-per-function': [
      'warn',
      { max: 100, skipBlankLines: true, skipComments: true, IIFEs: true },
    ],
    'new-cap': 'off',
    '@babel/new-cap': 'error',
    'no-alert': 'error',
    'no-array-constructor': 'error',
    'no-bitwise': 'error',
    'no-console': ['error', { allow: ['info', 'warn', 'error'] }],
    'no-div-regex': 'error',
    'no-empty-function': 'error',
    'no-eval': 'error',
    'no-extend-native': 'error',
    'no-floating-decimal': 'error',
    'no-implied-eval': 'error',
    'no-invalid-this': 'off',
    '@babel/no-invalid-this': 'error',
    'no-loop-func': 'error',
    'no-mixed-operators': [
      'error',
      {
        groups: [
          ['==', '!=', '===', '!==', '>', '>=', '<', '<='],
          ['&&', '||'],
          ['in', 'instanceof'],
        ],
        allowSamePrecedence: true,
      },
    ],
    'no-multi-str': 'error',
    'no-new': 'error',
    'no-new-object': 'error',
    'no-new-wrappers': 'error',
    'no-param-reassign': 'error',
    'no-return-assign': 'error',
    'no-return-await': 'error',
    'no-shadow': 'error',
    'no-throw-literal': 'error',
    'no-undef-init': 'error',
    'no-unused-expressions': 'off',
    '@babel/no-unused-expressions': [
      'error',
      {
        allowShortCircuit: true,
        allowTernary: false,
        allowTaggedTemplates: true,
      },
    ],
    'no-useless-computed-key': 'error',
    'no-useless-concat': 'error',
    'no-useless-constructor': 'error',
    'no-useless-rename': 'error',
    'no-useless-return': 'error',
    'no-var': 'error',
    'object-shorthand': 'error',
    'prefer-arrow-callback': 'error',
    'prefer-const': 'error',
    'prefer-object-spread': 'error',
    'prefer-promise-reject-errors': 'error',
    'prefer-regex-literals': 'error',
    'prefer-rest-params': 'error',
    'prefer-spread': 'error',
    'spaced-comment': 'error',

    // Plugin: ESLint Comments
    'eslint-comments/disable-enable-pair': 'off',
    'eslint-comments/no-unlimited-disable': 'off',

    // Plugin: File Names
    'filenames/match-regex': ['error', '^[\\[\\]_a-zA-Z0-9.-]+$'],

    // Plugin: Promise
    'promise/always-return': 'off',
    'promise/catch-or-return': 'off',

    // Plugin: Simple Import Sort
    'sort-imports': 'off',
    'import/order': 'off',
    'simple-import-sort/imports': 'warn',
    'simple-import-sort/exports': 'warn',
  },
  overrides: [
    {
      files: ['*.ts', '*.d.ts', '*.tsx'],
      parser: require.resolve('@typescript-eslint/parser'),
      parserOptions: {
        project: ['**/tsconfig.json'],
        sourceType: 'module',
      },
      plugins: ['@typescript-eslint/eslint-plugin'],
      extends: [
        'plugin:@typescript-eslint/eslint-plugin/recommended',
        'plugin:eslint-plugin-import/typescript',
      ],
      rules: {
        // Supported Rules
        camelcase: 'off',
        '@typescript-eslint/naming-convention': [
          'error',
          {
            selector: 'default',
            format: ['camelCase', 'PascalCase', 'UPPER_CASE', 'snake_case'],
            leadingUnderscore: 'allowSingleOrDouble',
            trailingUnderscore: 'forbid',
          },
          {
            selector: [
              'classProperty',
              'objectLiteralProperty',
              'typeProperty',
              'classMethod',
              'objectLiteralMethod',
              'typeMethod',
              'accessor',
              'enumMember',
            ],
            format: null,
            modifiers: ['requiresQuotes'],
          },
          {
            selector: 'typeLike',
            format: ['PascalCase'],
          },
        ],
        '@typescript-eslint/no-confusing-non-null-assertion': 'error',
        '@typescript-eslint/no-duplicate-enum-values': 'error',
        '@typescript-eslint/no-inferrable-types': [
          'error',
          { ignoreProperties: true },
        ],
        '@typescript-eslint/no-non-null-asserted-nullish-coalescing': 'error',
        '@typescript-eslint/prefer-enum-initializers': 'error',
        '@typescript-eslint/prefer-for-of': 'error',
        '@typescript-eslint/prefer-literal-enum-member': 'error',
        '@typescript-eslint/prefer-optional-chain': 'warn',
        '@typescript-eslint/typedef': [
          'error',
          {
            arrayDestructuring: false,
            arrowParameter: false,
            memberVariableDeclaration: true,
            objectDestructuring: false,
            parameter: false,
            propertyDeclaration: true,
            variableDeclaration: false,
            variableDeclarationIgnoreFunction: true,
          },
        ],

        // Extension Rules
        'default-param-last': 'off',
        '@typescript-eslint/default-param-last': 'error',

        'dot-notation': 'off',
        '@typescript-eslint/dot-notation': 'warn',

        'no-dupe-class-members': 'off',
        '@typescript-eslint/no-dupe-class-members': 'error',

        'no-duplicate-imports': 'off',
        '@typescript-eslint/no-duplicate-imports': 'error',

        'no-invalid-this': 'off',
        '@babel/no-invalid-this': 'off',
        '@typescript-eslint/no-invalid-this': 'error',

        'no-loop-func': 'off',
        '@typescript-eslint/no-loop-func': 'error',

        'no-redeclare': 'off',
        '@typescript-eslint/no-redeclare': 'error',

        'no-shadow': 'off',
        '@typescript-eslint/no-shadow': 'error',

        'no-throw-literal': 'off',
        '@typescript-eslint/no-throw-literal': 'error',

        'no-unused-expressions': 'off',
        '@babel/no-unused-expressions': 'off',
        '@typescript-eslint/no-unused-expressions': [
          'error',
          {
            allowShortCircuit: true,
            allowTernary: false,
            allowTaggedTemplates: true,
          },
        ],

        '@typescript-eslint/no-unused-vars': [
          'error',
          {
            vars: 'all',
            args: 'after-used',
            ignoreRestSiblings: true,
            argsIgnorePattern: '^_',
            caughtErrors: 'none',
          },
        ],

        'no-use-before-define': 'off',
        '@typescript-eslint/no-use-before-define': [
          'error',
          { functions: false, classes: false, variables: false },
        ],

        'no-useless-constructor': 'off',
        '@typescript-eslint/no-useless-constructor': 'error',

        'no-return-await': 'off',
        '@typescript-eslint/return-await': 'error',

        // Import
        'import/no-unresolved': 'off',
      },
    },
  ],
};
